#dock {
    border-radius: 9px;
    background-color: rgba(0,0,0,0.9);
    border-width: 2px;
    border-color: #5f5f5f;
}
 /* Panel */
.dock-app {
    padding: 4px;
    width: 70px;
    height: 70px;        
    border-radius: 4px;
    transition-duration: 100;
}

.dock-app.running {
    padding: 3px;
    border: 1px solid #181818;
    background-gradient-direction: vertical;
    background-gradient-start: #3d3d3d;
    background-gradient-end: #181818;
}

.dock-app.selected {
    padding: 3px;
    border: 1px solid #666666;
}

.dock-app.focused {
    padding: 3px;
    border: 1px solid #5f5f5f;
    background-gradient-direction: vertical;
    background-gradient-start: rgba(61,61,61,0.8);
    background-gradient-end: rgba(24,24,24,0.2);
}

.dock-app:hover {
    padding: 3px;
    border: 1px solid #666666;
    background-gradient-direction: vertical;
    background-gradient-start: rgba(61,61,61,0.8);
    background-gradient-end: rgba(24,24,24,0.2);
    transition-duration: 100;
}

.dock-app:active {
    padding: 3px;
    background-color: #1e1e1e;
    border: 1px solid #5f5f5f;
}

.dock-menu {
    font-size: 12px
}
